#pragma once


#include <types.h>
#include <std/string.h>
#include <std/inline.h>
#include <drivers/ide/ide.h>
#include <psf.h>

/* Hardware text mode color constants. */
enum vga_color {
	VGA_COLOR_BLACK = 0,
	VGA_COLOR_BLUE = 1,
	VGA_COLOR_GREEN = 2,
	VGA_COLOR_CYAN = 3,
	VGA_COLOR_RED = 4,
	VGA_COLOR_MAGENTA = 5,
	VGA_COLOR_BROWN = 6,
	VGA_COLOR_LIGHT_GREY = 7,
	VGA_COLOR_DARK_GREY = 8,
	VGA_COLOR_LIGHT_BLUE = 9,
	VGA_COLOR_LIGHT_GREEN = 10,
	VGA_COLOR_LIGHT_CYAN = 11,
	VGA_COLOR_LIGHT_RED = 12,
	VGA_COLOR_LIGHT_MAGENTA = 13,
	VGA_COLOR_LIGHT_BROWN = 14,
	VGA_COLOR_WHITE = 15,
};

static inline u8 vga_entry_color(const enum vga_color fg, const enum vga_color bg) {
	return fg | (bg << 4);
}

static inline u16 vga_entry(const u8 uc, const u8 color) {
	return (u16) uc | (u16) color << 8;
}

static usize terminal_width;
static usize terminal_height;

extern usize terminal_row;
extern usize terminal_column;
extern u8 terminal_color;
extern u16 *terminal_buffer;

void terminal_clear();
int terminal_initialize(framebuffer_t *terminal_fb);
void terminal_setcolor(u8 color);
void terminal_clearline(usize line);
void terminal_clearlines(usize from, usize to);
void terminal_updatecursor();
void terminal_checknewline();
void terminal_putentryat(char c, u8 color, usize x, usize y);
void terminal_putchar(char c);
void terminal_write(const char* data, usize size);

static inline void terminal_writestring(const char *data) {
	terminal_write(data, strlen(data));
}

static inline void terminal_writeline(const char *data) {
	terminal_writestring(data);
	terminal_putchar('\n');
}
